==> fastec_3.1.12_23.tar.xz: usr/local/include/fastec/fas_log.h <==
/*!
 * @file fas_log.h
 * @brief Fastec Error and Debug Logging library.
 *
 * Copyright 2015 (c) Fastec Imaging as an unpublished work.
 * All Rights Reserved.
 *
 * The information contained herein is confidential property of
 * Fastec Imaging. The use, copying, transfer or disclosure of such
 * information is prohibited except by express written agreement
 * with Fastec Imaging.
 */

#ifndef FAS_LOG_H_
#define FAS_LOG_H_
#pragma once

#if !defined(_WIN32)
#include <syslog.h>
#else
// These would normally reside in the syslog.h file.
#define LOG_EMERG   0   /* system is unusable */
#define LOG_ALERT   1   /* action must be taken immediately */
#define LOG_CRIT    2   /* critical conditions */
#define LOG_ERR     3   /* error conditions */
#define LOG_WARNING 4   /* warning conditions */
#define LOG_NOTICE  5   /* normal but significant condition */
#define LOG_INFO    6   /* informational */
#define LOG_DEBUG   7   /* debug-level messages */
#endif  /* _WIN32 */

#include <sstream>
#include <thread>

// Note that the syntax for variadic arguments is slightly different between
// Windows and Linux.
#if defined(_WIN32) && !defined(__MINGW32__)
#define LOG_TH(fmt, args, ...)                                        \
    do                                                                \
    {                                                                 \
        std::stringstream tid;                                        \
        tid << "[" << std::this_thread::get_id() << "] " << __FILE__; \
        tid << ":" << __FUNCTION__;                                   \
        fiLogTag(tid.str().c_str(), __LINE__, fmt, ##args);           \
    } while (0)
#else
#define LOG_TH(fmt, args...)                                          \
    do                                                                \
    {                                                                 \
        std::stringstream tid;                                        \
        tid << "[" << std::this_thread::get_id() << "] " << __FILE__; \
        tid << ":" << __FUNCTION__;                                   \
        fiLogTag(tid.str().c_str(), __LINE__, fmt, ##args);           \
    } while (0)
#endif

// To avoid compiling in the PDEBUG messages,
// #define DISABLE_PDEBUG before including log.h.
#ifdef DISABLE_PDEBUG
#define PDEBUG(level, fmt, args...) {}
#define PDEBUG_ENABLED(level) 0
#else
#undef PDEBUG             // undef it, just in case.

#if defined(_WIN32) && !defined(__MINGW32__)
#define PDEBUG(level, fmt, args, ...) \
    do { \
    if ( level & verbosity ) { \
        debug_log( fmt "\n", __VA_ARGS__ ); \
        } \
        } while (0)
#else
#define PDEBUG(level, fmt, args...) \
    do { \
    if ( level & verbosity ) { \
        debug_log( fmt "\n", ##args ); \
    } \
    } while (0)
#endif // !_WIN32

#define PDEBUG_ENABLED(level) ((( level ) & verbosity ) ? true : false )
#endif

/**
 * @brief Add a tagged, formatted, message to the logger output(s)
 *
 * This macro will send a formatted message to the currently configured
 * logger outputs. The tag will include the LOG() macro call source file
 * and line number, as well as a timestamp.
 *
 * @param fmt     printf()-style format string
 * @param args... Zero or more arguments for the format string
 */
#if defined(_WIN32) && !defined(__MINGW32__)
#  define LOG(fmt, ...) fiLogTag(__FILE__, __LINE__, fmt, __VA_ARGS__)
#else
#  define LOG(fmt, args...) fiLogTag(__FILE__, __LINE__, fmt, ##args)
#endif

/**
 * @brief Add a formatted message to the logger output(s)
 *
 * This macro will send a formatted message to the currently configured
 * logger outputs. The logger will not prepend and tag.
 *
 * @param fmt     printf()-style format string
 * @param args... Zero or more arguments for the format string
 */
#if defined(_WIN32) && !defined(__MINGW32__)
#  define LOG_NOTAG(fmt, ...) fiLog(fmt, __VA_ARGS__)
#else
#  define LOG_NOTAG(fmt, args...) fiLog(fmt, ##args)
#endif

// Function for PDEBUG logging.
extern void debug_log(const char *fmt, ...);

// Verbosity global variable for PDEBUG choices.
extern unsigned verbosity;

//-----------------------------------------------------------------------------
// Error logging functions.
//-----------------------------------------------------------------------------

//! Log an error that "should never happen" in a way that helps with debug.
#define LOG_UNLIKELY_ERRNO(f, l, e) fi_error_log_unlikely(f, l, e)
extern void fi_error_log_unlikely(const char *func, int line, int errno_value);

//! Normal error logging.
extern void fi_log(unsigned severity, const char *fmt, ...);

/** @name Support functions for logging macros @{*/

/**
 * @brief Format an un-tagged log message
 *
 * @param fmt   printf()-style format string, followed by zero or more arguments
 */
extern void fiLog(const char *fmt, ...);

/**
 * @brief Format a log message, tagged with timestamp, file name, and line number
 *
 * @param file  File name (generally supplied by the __FILE__ macro)
 * @param line  Line number (generally supplied by the __LINE__ macro)
 * @param fmt   printf()-style format string, followed by zero or more arguments
 */
extern void fiLogTag(const char *file, int line, const char *fmt, ...);

/** @name Control/Configuration Interface to Logger @{*/

/**
 * @brief Enable/disable logger output to syslog
 *
 * Log output may be optionally copied to syslog. When enabled, each line of
 * output sent to the logger (fi_log()) will become a separate timstamped/tagged
 * line in the systelog otuput. (Syselog does not have a provision for multi-line
 * entries, as may be produced by fi_log() calls.)
 *
 * @param enable  '1' (enable syslog output) or '0' (disable syslog output)
 */
extern void loggerUseSyslog(int enable);

/**
 * @brief Enable/disable logger output to stdout
 *
 * Log output may be optionally copied to a console, via the current process stdout.
 * This is in addition to any console output that may be enabled through syslog. It
 * is probably not useful to direct logger output to the console through both means.
 *
 * Log output sent to the console will not have any additional timestamps or tags
 * added. Multi-line log output sent through fi_log() are added 'atomically'
 * to the console.
 *
 * @param enable  '1' (enable stdout output) or '0' (disable stdout output)
 */
extern void loggerUseStdout(int enable);

/**
 * @brief Enable/disable logger output to 'private' log file
 *
 * When enabled (supplied with a non-NULL filename), logger output is appended
 * to the specified file. Supplying a NULL filename will close any log file
 * currently open, and stop sending logger output there.
 *
 * Log output sent to the file will not have any additional timestamps or tags
 * added. Multi-line log output sent through fi_log() are added 'atomically'
 * to the file.
 *
 * NOTE: Changing the log file name after it has been set once is not supported.
 * This function is designed to be called once and only once with a non-NULL
 * argument.
 *
 * @param filename  Pathname for log file
 * @return 0 if file successfully opened for logging. Non-zero E* (errno), or
 *           -1 for error.
 */
extern int loggerFile(const char *filename);

/**
 * @brief Re-open the logger output file
 *
 * This function is intended to support log file rotation via the logrotate(8)
 * utility. If a private log file has been enable (via loggerFile()), the file
 * is closed and re-opened before the next logger message is added to the file.
 * Client programs have to arrange to call this function after logrotate(8) has
 * renamed (rotated) an old log file, for example by catching a suitable signal.
 *
 * In order for all this to work, the logrotate(8) configuration will also need
 * to be set up to properly manage the private log files.
 */
extern void loggerReopen();
/**@}*/

#endif /* !FAS_LOG_H_ */

==> fastec_3.2.3_43.tar.xz: usr/local/include/fastec/fas_log.h <==
/*!
 * @file fas_log.h
 * @brief Fastec Error and Debug Logging library.
 *
 * Copyright 2015 (c) Fastec Imaging as an unpublished work.
 * All Rights Reserved.
 *
 * The information contained herein is confidential property of
 * Fastec Imaging. The use, copying, transfer or disclosure of such
 * information is prohibited except by express written agreement
 * with Fastec Imaging.
 */

#ifndef FAS_LOG_H_
#define FAS_LOG_H_
#pragma once

#if !defined(_WIN32)
#include <syslog.h>
#else
// These would normally reside in the syslog.h file.
#define LOG_EMERG   0   /* system is unusable */
#define LOG_ALERT   1   /* action must be taken immediately */
#define LOG_CRIT    2   /* critical conditions */
#define LOG_ERR     3   /* error conditions */
#define LOG_WARNING 4   /* warning conditions */
#define LOG_NOTICE  5   /* normal but significant condition */
#define LOG_INFO    6   /* informational */
#define LOG_DEBUG   7   /* debug-level messages */
#endif  /* _WIN32 */

#include <sstream>
#include <string>
#include <thread>

#include <fmt/printf.h>

// Use the libfmt library to create a string in a safe manner.
template <typename S, typename... Args>
std::string fmtToString(const S& format, const Args& ... args)
{
    std::string msg;

    try
    {
        msg = fmt::sprintf(format, args...);
    }
    catch (fmt::system_error& e)
    {
        msg = std::string("System Error: ") + e.what();
    }
    catch (fmt::format_error& e)
    {
        msg = std::string("Format Error: ") + e.what();
    }
    catch (...)
    {
        msg = std::string("Unknown exception!");
    }

    return msg;
}
template <typename S>
std::string fmtToString(const S& format)
{
    return std::string(format);
}


/**
 * @brief Format a log message, tagged with timestamp, file name, and line number
 *
 * @param file  File name (generally supplied by the __FILE__ macro)
 * @param line  Line number (generally supplied by the __LINE__ macro)
 * @param fmt   printf()-style format string, followed by zero or more arguments
 */
#define FI_LOG_TAG(file, line, fmt, ...) \
    logTagMsg(std::string(file), line, fmtToString(fmt, ##__VA_ARGS__)); \

extern void logTagMsg(std::string file, int line, std::string msg);

// Note that the syntax for variadic arguments is slightly different between
// Windows and Linux.
#if defined(_WIN32) && !defined(__MINGW32__)
#define LOG_TH(fmt, args, ...)                                        \
    do                                                                \
    {                                                                 \
        std::stringstream tid;                                        \
        tid << "[" << std::this_thread::get_id() << "] " << __FILE__; \
        tid << ":" << __FUNCTION__;                                   \
        FI_LOG_TAG(tid.str().c_str(), __LINE__, fmt, ##args);           \
    } while (0)
#else
#define LOG_TH(fmt, args...)                                          \
    do                                                                \
    {                                                                 \
        std::stringstream tid;                                        \
        tid << "[" << std::this_thread::get_id() << "] " << __FILE__; \
        tid << ":" << __FUNCTION__;                                   \
        FI_LOG_TAG(tid.str().c_str(), __LINE__, fmt, ##args);           \
    } while (0)
#endif

// To avoid compiling in the PDEBUG messages,
// #define DISABLE_PDEBUG before including log.h.
#ifdef DISABLE_PDEBUG
#define PDEBUG(level, fmt, args...) {}
#define PDEBUG_ENABLED(level) 0
#else
#undef PDEBUG             // undef it, just in case.

#if defined(_WIN32) && !defined(__MINGW32__)
#define PDEBUG(level, fmt, args, ...) \
    do { \
    if ( level & verbosity ) { \
        debug_log( fmt "\n", __VA_ARGS__ ); \
        } \
        } while (0)
#else
#define PDEBUG(level, fmt, args...) \
    do { \
    if ( level & verbosity ) { \
        debug_log( fmt "\n", ##args ); \
    } \
    } while (0)
#endif // !_WIN32

#define PDEBUG_ENABLED(level) ((( level ) & verbosity ) ? true : false )
#endif

/**
 * @brief Add a tagged, formatted, message to the logger output(s)
 *
 * This macro will send a formatted message to the currently configured
 * logger outputs. The tag will include the LOG() macro call source file
 * and line number, as well as a timestamp.
 *
 * @param fmt     printf()-style format string
 * @param args... Zero or more arguments for the format string
 */
#if defined(_WIN32) && !defined(__MINGW32__)
#  define LOG(fmt, ...) FI_LOG_TAG(__FILE__, __LINE__, fmt, __VA_ARGS__)
#else
#  define LOG(fmt, args...) FI_LOG_TAG(__FILE__, __LINE__, fmt, ##args)
#endif

/**
 * @brief Add a formatted message to the logger output(s)
 *
 * This macro will send a formatted message to the currently configured
 * logger outputs. The logger will not prepend and tag.
 *
 * @param fmt     printf()-style format string
 * @param args... Zero or more arguments for the format string
 */
#if defined(_WIN32) && !defined(__MINGW32__)
#  define LOG_NOTAG(fmt, ...) fiLog(fmt, ##__VA_ARGS__)
#else
#  define LOG_NOTAG(fmt, args...) fiLog(fmt, ##args)
#endif

// Function for PDEBUG logging.
#define debug_log(fmt, ...) \
    logMsg(LOG_DEBUG, fmtToString(fmt, __VA_ARGS__))

// The function to write the message out.
extern void logMsg(unsigned severity, std::string msg);

// Verbosity global variable for PDEBUG choices.
extern unsigned verbosity;

//-----------------------------------------------------------------------------
// Error logging functions.
//-----------------------------------------------------------------------------

//! Log an error that "should never happen" in a way that helps with debug.
#define LOG_UNLIKELY_ERRNO(f, l, e) fi_error_log_unlikely(f, l, e)
extern void fi_error_log_unlikely(const char *func, int line, int errno_value);

//! Normal error logging.
#define fi_log(severity, fmt, ...) \
    logMsg(severity, fmtToString(fmt, ##__VA_ARGS__))

/** @name Support functions for logging macros @{*/

/**
 * @brief Format an un-tagged log message
 *
 * @param fmt   printf()-style format string, followed by zero or more arguments
 */
#define fiLog(fmt, ...) \
    logMsg(LOG_DEBUG, fmtToString(fmt, ##__VA_ARGS__))

/** @name Control/Configuration Interface to Logger @{*/

/**
 * @brief Enable/disable logger output to syslog
 *
 * Log output may be optionally copied to syslog. When enabled, each line of
 * output sent to the logger (fi_log()) will become a separate timstamped/tagged
 * line in the systelog otuput. (Syselog does not have a provision for multi-line
 * entries, as may be produced by fi_log() calls.)
 *
 * @param enable  '1' (enable syslog output) or '0' (disable syslog output)
 */
extern void loggerUseSyslog(int enable);

/**
 * @brief Enable/disable logger output to stdout
 *
 * Log output may be optionally copied to a console, via the current process stdout.
 * This is in addition to any console output that may be enabled through syslog. It
 * is probably not useful to direct logger output to the console through both means.
 *
 * Log output sent to the console will not have any additional timestamps or tags
 * added. Multi-line log output sent through fi_log() are added 'atomically'
 * to the console.
 *
 * @param enable  '1' (enable stdout output) or '0' (disable stdout output)
 */
extern void loggerUseStdout(int enable);

/**
 * @brief Enable/disable logger output to 'private' log file
 *
 * When enabled (supplied with a non-NULL filename), logger output is appended
 * to the specified file. Supplying a NULL filename will close any log file
 * currently open, and stop sending logger output there.
 *
 * Log output sent to the file will not have any additional timestamps or tags
 * added. Multi-line log output sent through fi_log() are added 'atomically'
 * to the file.
 *
 * NOTE: Changing the log file name after it has been set once is not supported.
 * This function is designed to be called once and only once with a non-NULL
 * argument.
 *
 * @param filename  Pathname for log file
 * @return 0 if file successfully opened for logging. Non-zero E* (errno), or
 *           -1 for error.
 */
extern int loggerFile(const char *filename);

/**
 * @brief Re-open the logger output file
 *
 * This function is intended to support log file rotation via the logrotate(8)
 * utility. If a private log file has been enable (via loggerFile()), the file
 * is closed and re-opened before the next logger message is added to the file.
 * Client programs have to arrange to call this function after logrotate(8) has
 * renamed (rotated) an old log file, for example by catching a suitable signal.
 *
 * In order for all this to work, the logrotate(8) configuration will also need
 * to be set up to properly manage the private log files.
 */
extern void loggerReopen();
/**@}*/

#endif /* !FAS_LOG_H_ */
